#include "iptc.h"

#include <stdlib.h>
#include <string.h>


extern struct prefs_struct prefs;
extern unsigned char* rewrite_cache;
extern unsigned long int rewrite_cache_offset;


/* TODO: document the IPTC codes with a leading empty comment below (they are non-IIM 4.1) */
struct iptc_code_descr iptc_codes_descr[]=
{
	/* IPTC IIM 4.1 */
	/* envelope record */
	{	IPTC_MODEL_VERSION,										IPTC_DATASET_SPECIFIC,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"model version"  },
	{	IPTC_DESTINATION,										IPTC_DATASET_BINARY,	IPTC_DATASET_REPEATABLE,	IPTC_DATASET_MAX,	1024,	"destination"  },
	{	IPTC_FILE_FORMAT,										IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"file format"  },
	{	IPTC_FILE_FORMAT_VERSION,								IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"file format version"  },
	{	IPTC_SERVICE_ID,										IPTC_DATASET_TEXT,		IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	10,		"service ID"  },
	{	IPTC_ENVELOPE_NUMBER,									IPTC_DATASET_TEXT,		IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	8,		"envelope number"  },
	{	IPTC_PRODUCT_ID,										IPTC_DATASET_TEXT,		IPTC_DATASET_REPEATABLE,	IPTC_DATASET_MAX,	32,		"product ID"  },
	{	IPTC_ENVELOPE_PRIORITY,									IPTC_DATASET_TEXT,		IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	1,		"envelope priority"  },
	{	IPTC_DATE_SENT,											IPTC_DATASET_DATE,		IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	8,		"date sent"  },
	{	IPTC_TIME_SENT,											IPTC_DATASET_TIME,		IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	11,		"time sent"  },
	{	IPTC_CODED_CHARACTER_SET,								IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	32,		"character set"  },
	{	IPTC_UNO,												IPTC_DATASET_TEXT,		IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	80,		"unique name of object"  },
	{	IPTC_ARM_ID,											IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"abstract relationship method ID"  },
	{	IPTC_ARM_VERSION,										IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"abstract relationship method version"  },
	/* application record */
	{	IPTC_APP_RECORD_VERSION,								IPTC_DATASET_SPECIFIC,  IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED, 2,		"record version"	},
	{	IPTC_APP_OBJECT_TYPE_REFERENCE,					    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	67,		"object type reference" },
	{	IPTC_APP_OBJECT_ATTRIBUTE_REFERENCE,			    	IPTC_DATASET_TEXT,  	IPTC_DATASET_REPEATABLE,	IPTC_DATASET_MAX,	68,		"object attribute reference"	},
	{	IPTC_APP_OBJECT_NAME,							    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	64,		"object name (title)"	},
	{	IPTC_APP_EDIT_STATUS,							    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	64,		"edit status"	},
	{	IPTC_APP_EDITORIAL_UPDATE,						    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"editorial update"  },
	{	IPTC_APP_URGENCY,								    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	1,		"urgency"	},
	{	IPTC_APP_SUBJECT_REFERENCE,						    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	236,	"subject reference" },
	{	IPTC_APP_CATEGORY,								    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	3,		"category"  },
/**/{	IPTC_APP_CATEGORY_DESCRIPTION,							IPTC_DATASET_TEXT,  	IPTC_DATASET_REPEATABLE,	IPTC_DATASET_MAX,	0,		"category description"  },
	{	IPTC_APP_SUPPLEMENTAL_CATEGORY,							IPTC_DATASET_TEXT,  	IPTC_DATASET_REPEATABLE,	IPTC_DATASET_MAX,	32,		"supplemental category"	},
/**/{	IPTC_APP_CATEGORY_CODE,									IPTC_DATASET_TEXT,  	IPTC_DATASET_REPEATABLE,	IPTC_DATASET_MAX,	0,		"category code" },
	{	IPTC_APP_FIXTURE_IDENTIFIER,					    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	32,		"fixture identifier"	},
/**/{	IPTC_APP_NUM,									    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	0,		"num"	},
	{	IPTC_APP_KEYWORDS,								    	IPTC_DATASET_TEXT,  	IPTC_DATASET_REPEATABLE,	IPTC_DATASET_MAX,	64,		"keywords"  },
	{	IPTC_APP_CONTENT_LOCATION_CODE,					    	IPTC_DATASET_TEXT,		IPTC_DATASET_REPEATABLE,	IPTC_DATASET_FIXED,	3,		"content location code" },
	{	IPTC_APP_CONTENT_LOCATION_NAME,					    	IPTC_DATASET_TEXT,		IPTC_DATASET_REPEATABLE,	IPTC_DATASET_MAX,	64,		"content location name" },
	{	IPTC_APP_RELEASE_DATE,							    	IPTC_DATASET_DATE,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	8,		"release date"  },
	{	IPTC_APP_RELEASE_TIME,							    	IPTC_DATASET_TIME,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	11,		"release time"  },
	{	IPTC_APP_EXPIRATION_DATE,						    	IPTC_DATASET_DATE,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	8,		"expiration date"	},
	{	IPTC_APP_EXPIRATION_TIME,						    	IPTC_DATASET_TIME,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	1,		"expiration time"	},
	{	IPTC_APP_SPECIAL_INSTRUCTIONS,					    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	256,	"special instructions"  },
	{	IPTC_APP_ACTION_ADVISED,						    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"action advised"	},
	{	IPTC_APP_REFERENCE_SERVICE,						    	IPTC_DATASET_TEXT,  	IPTC_DATASET_REPEATABLE,	IPTC_DATASET_MAX,	10,		"reference service" },
	{	IPTC_APP_REFERENCE_DATE,						    	IPTC_DATASET_DATE,  	IPTC_DATASET_REPEATABLE,	IPTC_DATASET_FIXED,	8,		"reference date"	},
	{	IPTC_APP_REFERENCE_NUMBER,						    	IPTC_DATASET_TEXT,  	IPTC_DATASET_REPEATABLE,	IPTC_DATASET_MAX,	8,		"reference number"  },
	{	IPTC_APP_DATE_CREATED,							    	IPTC_DATASET_DATE,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	8,		"date created"  },
/**/{	IPTC_APP_ARCHIVE_DATE,									IPTC_DATASET_DATE,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	8,		"archive date"  },
	{	IPTC_APP_TIME_CREATED,							    	IPTC_DATASET_TIME,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	11,		"time created"  },
	{	IPTC_APP_DIGITAL_CREATION_DATE,					    	IPTC_DATASET_DATE,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	8,		"digital creation date" },
	{	IPTC_APP_DIGITAL_CREATION_TIME,					    	IPTC_DATASET_TIME,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	11,		"digital creation time" },
	{	IPTC_APP_ORIGINATING_PROGRAM,					    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	32,		"originating program"	},
	{	IPTC_APP_PROGRAM_VERSION,						    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	10,		"program version"	},
	{	IPTC_APP_OBJECT_CYCLE,							    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	1,		"object cycle"  },
	{	IPTC_APP_BYLINE,								    	IPTC_DATASET_TEXT,  	IPTC_DATASET_REPEATABLE,	IPTC_DATASET_MAX,	32,		"byline (author)"	},
	{	IPTC_APP_BYLINE_TITLE,							    	IPTC_DATASET_TEXT,  	IPTC_DATASET_REPEATABLE,	IPTC_DATASET_MAX,	32,		"byline title (author position)"	},
/**/{	IPTC_APP_TITLE_ORIGINAL,								IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	0,		"title original"	},
/**/{	IPTC_APP_PRODUCTOR,										IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	0,		"productor" },
/**/{	IPTC_APP_ACTOR,											IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	0,		"actor" },
	{	IPTC_APP_CITY,									    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	32,		"city"  },
	{	IPTC_APP_SUBLOCATION,							    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	32,		"sub-location"  },
	{	IPTC_APP_PROVINCE_STATE,						    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	32,		"state/province"	},
	{	IPTC_APP_COUNTRY_CODE,							    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	3,		"country/primary location code"  },
	{	IPTC_APP_COUNTRY,								    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	64,		"country/primary location name"  },
	{	IPTC_APP_ORIGINAL_TRANSMISSION_REFERENCE,		    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	32,		"original transmission reference"	},
	{	IPTC_APP_HEADLINE,								    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	256,	"headline"  },
	{	IPTC_APP_CREDIT,								    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	32,		"credit"	},
/**/{	IPTC_APP_PHOTOGRAPHER_NUMBER,							IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	0,		"photographer number"	},
	{	IPTC_APP_SOURCE,								    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	32,		"source"	},
	{	IPTC_APP_COPYRIGHT_NOTICE,						    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	128,	"copyright notice"  },
	{	IPTC_APP_CONTACT,								    	IPTC_DATASET_TEXT,  	IPTC_DATASET_REPEATABLE,	IPTC_DATASET_MAX,	128,	"contact"	},
	{	IPTC_APP_CAPTION_ABSTRACT,						    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	2000,	"caption/abstract"	},
/**/{	IPTC_APP_PHOTOGRAPH_NUMBER,								IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	0,		"photograph number" },
	{	IPTC_APP_WRITER_EDITOR,									IPTC_DATASET_TEXT,  	IPTC_DATASET_REPEATABLE,	IPTC_DATASET_MAX,	32,		"writer/editor"	},
/**/{	IPTC_APP_SERIAL_NUMBER,									IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	0,		"serial number" },
	{	IPTC_APP_RASTERIZED_CAPTION,					    	IPTC_DATASET_BINARY,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	7360,	"rasterized caption"	},
	{	IPTC_APP_IMAGE_TYPE,							    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"image type"	},
	{	IPTC_APP_IMAGE_ORIENTATION,						    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	1,		"image orientation" },
/**/{	IPTC_APP_REFERENCE,										IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	0,		"reference"  },
	{	IPTC_APP_LANGUAGE_IDENTIFIER,					    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	3,		"language identifier"	},
	{	IPTC_APP_AUDIO_TYPE,							    	IPTC_DATASET_TEXT,  	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"audio type"	},
	{	IPTC_APP_AUDIO_SAMPLING_RATE,					    	IPTC_DATASET_TEXT,		IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	6,		"audio sampling rate"	},
	{	IPTC_APP_AUDIO_SAMPLING_RESOLUTION,				    	IPTC_DATASET_TEXT,		IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"audio sampling resolution"	},
	{	IPTC_APP_AUDIO_DURATION,						    	IPTC_DATASET_TEXT,		IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	6,		"audio duration"	},
	{	IPTC_APP_AUDIO_OUTCUE,							    	IPTC_DATASET_TEXT,		IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	64,		"audio outcue"	},
	{	IPTC_APP_OBJECTDATA_PREVIEW_FILE_FORMAT,		    	IPTC_DATASET_TEXT,		IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"objectdata preview file format"	},
	{	IPTC_APP_OBJECTDATA_PREVIEW_FILE_FORMAT_VERSION,    	IPTC_DATASET_TEXT,		IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"objectdata preview file format version"	},
	{	IPTC_APP_OBJECTDATA_PREVIEW_PREVIEW_DATA,		    	IPTC_DATASET_TEXT,		IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	256000,	"objectdata preview data"	},
	/* newsphoto */
	{	IPTC_NP_RECORD_VERSION,									IPTC_DATASET_SPECIFIC,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"record version"	},
	{	IPTC_NP_PICTURE_NUMBER,									IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	16,		"picture numver"	},
	{	IPTC_NP_PIXELS_PER_LINE,								IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"pixels per line"	},
	{	IPTC_NP_NUMBER_OF_LINES,								IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"number of lines"	},
	{	IPTC_NP_PIXEL_SIZE_IN_SCANNING_DIRECTION,				IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"pixel size in scanning direction"	},
	{	IPTC_NP_PIXEL_SIZE_PERPENDICULAR_TO_SCANNING_DIRECTION,	IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"pixel size perpendicular to scanning direction"	},
	{	IPTC_NP_SUPPLEMENTARY_TYPE,								IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	1,		"supplementary type"	},
	{	IPTC_NP_COLOUR_REPRESENTATION,							IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"colour representation"	},
	{	IPTC_NP_INTERCHANGE_COLOUR_SPACE,						IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	1,		"inter-change colour space"	},
	{	IPTC_NP_COLOUR_SEQUENCE,								IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	4,		"colour sequence"	},
	{	IPTC_NP_ICC_INPUT_COLOUR_PROFILE,						IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	524288,	"ICC input colour profile"	},
	{	IPTC_NP_COLOUR_CALIBRATION_MATRIX_TABLE,				IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	0xffff,	"colour calibration matrix table"	},
	{	IPTC_NP_LOOKUP_TABLE,									IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	131072,	"lookup table"	},
	{	IPTC_NP_NUMBER_OF_INDEX_ENTRIES,						IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	2,		"number of index entries"	},
	{	IPTC_NP_COLOUR_PALETTE,									IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	524288,	"colour palette"	},
	{	IPTC_NP_NUMBER_OF_BITS_PER_SAMPLE,						IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	1,		"number of bits per sample"	},
	{	IPTC_NP_SAMPLING_STRUCTURE,								IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	1,		"sampling structure"	},
	{	IPTC_NP_SCANNING_DIRECTION,								IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	1,		"scanning direction"	},
	{	IPTC_NP_IMAGE_ROTATION,									IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	1,		"image rotation"	},
	{	IPTC_NP_DATA_COMPRESSION_METHOD,						IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	4,		"data compression method"	},
	{	IPTC_NP_QUANTISATION_METHOD,							IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	1,		"quantisation method"	},
	{	IPTC_NP_END_POINTS,										IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	0xffff,	"end points"	},
	{	IPTC_NP_EXCURSION_TOLERANCE,							IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	1,		"excursion tolerance"	},
	{	IPTC_NP_BITS_PER_COMPONENT,								IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	4,		"bits per component"	},
	{	IPTC_NP_MAXIMUM_DENSITY_RANGE,							IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"maximum density range"	},
	{	IPTC_NP_GAMMA_COMPENSATED_VALUE,						IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	2,		"gamma compensated value"	},
	/* pre-objectdata descriptor record */
 	{	IPTC_PRODD_SIZE_MODE,									IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_FIXED,	1,		"size mode"	},
 	{	IPTC_PRODD_MAX_SUBFILE_SIZE,							IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	4,		"max subfile size"	},
 	{	IPTC_PRODD_OBJECTDATA_SIZE_ANNOUNCED,					IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	4,		"objectdata size announced"	},
 	{	IPTC_PRODD_MAXIMAL_OBJECTDATA_SIZE,						IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	4,		"maximal objectdata size"	},
	/* objectdata descriptor record */
	/* here the 0xffffffff limit is the real limit (not because undocumented ) */
 	{	IPTC_ODD_SUBFILE,										IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	0xffffffff,	"subfile"	},
	/* post-objectdata descriptor record */
 	{	IPTC_POODD_CONFIRMED_OBJECT_SIZE,						IPTC_DATASET_BINARY,	IPTC_DATASET_UNIQUE,		IPTC_DATASET_MAX,	4,	"confirmed object size"	},
	/* reserved */
	{	IPTC_EOT, IPTC_DATASET_UNKNOWN, IPTC_DATASET_UNIQUE, IPTC_DATASET_MAX, 0, NULL }
};

struct iptc_filter_match** iptc_filter_table;
unsigned short int iptc_filter_table_size;
unsigned short int iptc_filter_table_filters;
bool iptc_filter_match_exclude;
unsigned char* iptc_extract_buffer;
unsigned long int iptc_extract_buffer_size;


/* IPTC-NAA IM4.1 */
/* attempts to recognize all IPTC codes, and display data of known ones */
void iptc_parse(unsigned char* buffer, const long int bytes_left, const bool iptc_expected,
		struct parser_result* result)
{
	struct iptc_code_descr* code_descr;
	unsigned short int* iptc_datasets_history;
	unsigned char* ptr;
	unsigned char* value;
	unsigned long int size;
	unsigned long int written_bytes;
	unsigned long int iptc_datasets;
	long int missing;
	long int bytes;
	unsigned short int code;
	unsigned short int length;
	unsigned char record;
	unsigned char last_record;
	unsigned char tag;
	bool ret;
	bool saw_warning_missing_record_version[256];
	bool saw_warning_duplicate_record_version[256];

	iptc_datasets=0;
	ptr=buffer;
	bytes=bytes_left;
	written_bytes=0;
	missing=0;
	ret=true;
	debug("IPTC parser: %ld byte(s) to parse", bytes);

	/* reset table of encountered datasets (to manage repeatable ones) */
	if (prefs.technical)
	{
		/* limit count to 65535 per-record occurrences */
		size=0x10000UL*sizeof(unsigned short int);
		iptc_datasets_history=malloc(size);
		if (!iptc_datasets_history)
			fatal_error("could not allocate dataset history buffer");
		memset(iptc_datasets_history, 0, size);
	}
	else
		iptc_datasets_history=NULL;
	memset(saw_warning_missing_record_version, false, 256*sizeof(bool));
	memset(saw_warning_duplicate_record_version, false, 256*sizeof(bool));

	/* records must be encountered following a numerical value order */
	last_record=0;

	while (bytes>0)
	{
		if (bytes<5)
		{
			ret=false;
			error("IPTC parser: record underrun (%ld byte(s) remaining, 5 or more expected)",
					bytes);
			break;
		}

		/* IPTC tag marker expected */
		tag=getbyte(ptr, 0);
		if ((tag!=IPTC_MARKER) && iptc_expected)
		{
			ret=false;
			error("IPTC parser: expected tag marker not matched");
			break;
		}
		
		/* IPTC code (record number + code_descr number) expected */
		record=getbyte(ptr, 1);
		if (prefs.technical && (record==0))
			warning("IPTC parser: illegal record number 0");
		/* check if records are encountered following a numerical value order */
		if (prefs.technical && (record<last_record))
			warning("IPTC parser: illegal record order (got 0x%02x, last one was 0x%02x)",
					record, last_record);
		last_record=record;
		code=(record<<8)|getbyte(ptr, 2);
		code_descr=iptc_match_code((const enum IPTC_CODE)code);
		if (!code_descr)
			warning("IPTC parser: unknown IPTC code 0x%04x", code);
		else
		{
			debug("IPTC parser: code 0x%04x (%s)", code, code_descr->label);
			/* update datasets history, check if this dataset occurrence is expected */
			if (	prefs.technical
				&&	(code_descr->repeatable==IPTC_DATASET_UNIQUE)
				&&	iptc_datasets_history[code])
				error("IPTC parser: dataset not repeatable but already found %u time(s), this new one should be ignored",
						iptc_datasets_history[code]);
		}
		if (prefs.technical)
		{
			/* some checks upon the IPTC code */
			/* is this a record version code, is it duplicate? */
			if (	!saw_warning_duplicate_record_version[record]
				&&	((code&0x00ff)==0)
				&&	(iptc_datasets_history[code]>0))
			{
				saw_warning_duplicate_record_version[record]=true;
				warning("IPTC parser: more than one record version found (code: 0x%04x)",
						code);
			}
			/* did we met a record version code for this record type yet? */
			if (	!saw_warning_missing_record_version[record]
				&&	((code&0x00ff)!=0)
				&&	(iptc_datasets_history[record]==0))
			{
				saw_warning_missing_record_version[record]=true;
				warning("IPTC parser: no record version code found yet (expected code: 0x%04x)",
						code&0xff00);
			}
			/* update IPTC code stats table */
			if (iptc_datasets_history[code]==0xffff)
				error("IPTC parser: datasets history overflow (0x%04x)", code);
			else
				iptc_datasets_history[code]++;
		}
		iptc_datasets++;

		/* IPTC dataset length */
		length=getword(ptr, 3);
		/* TODO: manage 32-bit lengths (newsphoto datasets) */
		/* warn if length doesn't match the expected one */
		if (prefs.technical && code_descr)
		{
			if (code_descr->fixed_length)
			{
				if (length!=code_descr->length)
					warning("IPTC parser: dataset 0x%04x: length (%lu byte(s)) doesn't match the expected length (%lu byte(s))",
							code, length, code_descr->length);
			}
			else
				if ((code_descr->length>0) && (length>code_descr->length))
					warning("IPTC parser: dataset 0x%04x: length (%lu byte(s)) exceeds the expected max length (%lu byte(s))",
							code, length, code_descr->length);
		}

		/* dump dataset header */
		if ((prefs.op==OP_DUMP) || (prefs.op==OP_DUMP_FULL))
		{
			printf("IPTC ");
#ifdef IPTC_SHOW_NUMERICAL_ID
			printf("%d:%03d", (code&0xff00U)>>8, code&0xffU);
#else
			printf("0x%04x", code);
#endif
			printf(" %s", code_descr?code_descr->label:"<unknown dataset>");
		}

		/* IPTC dataset value */
		missing=bytes-(1+2+2+length);
		if (missing<0)
		{
			message(prefs.fix?SEV_WARNING:SEV_ERROR, "IPTC parser: dataset 0x%04x: underrun (%ld byte(s) left, %ld expected)%s",
					code, bytes, 1+2+2+length, prefs.fix?" - about to be fixed":"");
			if (prefs.fix)
			{
				missing=abs(missing);
				prefs.context_fixes++;
				prefs.fixes++;
			}
			else
			{
				/* don't fix missing bytes, report an error */
				missing=0;
				ret=false;
			}
			if (bytes==0)
			{
				if ((prefs.op==OP_DUMP) || (prefs.op==OP_DUMP_FULL))
					printf("\n");
				break;
			}
			/* show remaining bytes even if dataset seems corrupted */
			length=bytes;
			length-=1+2+2;
		}
		else
			missing=0;
		value=malloc((length+missing+1)*sizeof(unsigned char));
		if (!value)
		{
			if ((prefs.op==OP_DUMP) || (prefs.op==OP_DUMP_FULL))
				printf("\n");
			fatal_error("IPTC parser: malloc(%lu) for dataset 0x%04x",
					(length+1)*sizeof(unsigned char), code);
		}
		memcpy(value, ptr+1+2+2, length);
		/* are there missing bytes to fix? */
		if (missing)
			/* if so, fill them with 0 */
			memset(value+(length*sizeof(unsigned char)), 0xBA,
					missing*sizeof(unsigned char));
		/* if the value is a string, it will be possible to use its value
		   as a 0-terminated string */
		value[length+missing]=0;

		/* conditional rewrite of the full dataset (incl. tag and header) */
		if (prefs.rewrite && iptc_filter_match(code))
		{
			written_bytes+=dump_rewrite(ptr, MIN(1+2+2+length, bytes));
			if (missing>0)
			{
				debug("IPTC parser: writing %ld byte(s) to fix the underrun",
						missing);
				written_bytes+=dump_rewrite(ptr+1+2+2+length, missing);
			}
		}

		/* skip dataset (incl. tag and header) */
		ptr+=1+2+2+length;
		bytes-=1+2+2+length;

		/* dump full dataset values */
		if (prefs.op==OP_DUMP_FULL)
		{
			printf(" ");
			if (!code_descr)
				dump_hexa(value, length);
			else
			{
				/* dataset value formatting */
				switch (code_descr->format)
				{
					case IPTC_DATASET_UNKNOWN:
					case IPTC_DATASET_BINARY:
						dump_hexa(value, length);
						break;

					case IPTC_DATASET_TEXT:
						printf("[%u byte(s)]: '%s'\n", length, value);
						break;

					case IPTC_DATASET_DATE:
						/* TODO: decode and use a date format instead */
						printf("[%u byte(s)]: '%s'\n", length, value);
						break;

					case IPTC_DATASET_TIME:
						/* TODO: decode and use a time format instead */
						printf("[%u byte(s)]: '%s'\n", length, value);
						break;

					case IPTC_DATASET_NUMERICAL:
						if (length==1)
							printf("[%u byte(s)]: %d", length, (char)getbyte(value, 0));
						else
						if (length==2)
							printf("[%u byte(s)]: %d", length, (short int)getword(value, 0));
						else
						if (length==4)
							printf("[%u byte(s)]: %ld", length, (long int)getlong(value, 0));
						else
						{
							printf("\n");
							warning("IPTC parser: dataset 0x%04x: don't know how to print numerical value (%u byte(s))\n",
									code, length);
						}
						break;

					case IPTC_DATASET_SPECIFIC:
						switch ((enum IPTC_CODE)code)
						{
							case IPTC_APP_RECORD_VERSION:
							{
								printf("[%d byte(s)]: %d.%d\n", length,
										getbyte(value, 1), getbyte(value, 0));
								break;
							}
							default:
								printf("\n");
								warning("IPTC parser: dataset 0x%04x: don't know how to print specific value (%u byte(s))\n",
										code, length);
								break;
						}
						break;

					default:
						break;
				}
			}
		}
		else
			if (prefs.op==OP_DUMP)
				printf("\n");

		free(value);
		value=NULL;
	}

	if (prefs.technical)
	{
		/* a posteriori check if record version code has been found when it should have been */
/*
		unsigned short int record_type;

		for (record_type=0x0100;
			record_type<=0x0300;
			record_type+=0x0100)
		{
			if (iptc_datasets_history[record_type]==0)
			{
				register unsigned short int dataset;

				for (dataset=1; dataset<=0xff; dataset++)
					if (iptc_datasets_history[record_type+dataset]>0)
					{
						warning("IPTC parser: no record version code found at all for record type 0x%02x",
								record_type>>8);
						break;
					}
			}
		}
*/
		register unsigned short int record_type;

		for (record_type=0x01; record_type<=0xff; record_type++)
			if (saw_warning_missing_record_version[record_type])
				warning("IPTC parser: no record version code found at all for record type 0x%02x",
						record_type);
		free(iptc_datasets_history);
		iptc_datasets_history=NULL;
	}
	debug("IPTC parser: %u dataset(s) found, total length: %lu byte(s)",
			iptc_datasets, bytes_left-bytes);
	if (prefs.rewrite)
		debug("IPTC parser: %lu byte(s) rewritten", written_bytes);
	result->parsed_bytes=bytes_left-bytes;
	result->written_bytes=written_bytes;
	result->ret=ret;

	if (prefs.extract_iptc)
	{
		if (missing>0)
		{
			if (!prefs.fix)
				error("IPTC parser: unexpected state: fixing not enabled but there are %ld byte(s) marked for fixing (extraction)",
						missing);
			else
				debug("IPTC parser: there are %ld byte(s) marked for fixing (extraction)",
						missing);
		}

		if (prefs.rewrite_cached)
		{
			/* copy cached rewritten ITPC chunk for later saving */
			iptc_extract_buffer_size=result->written_bytes;
			iptc_extract_buffer=(unsigned char*)malloc(iptc_extract_buffer_size*sizeof(unsigned char));
			if (!iptc_extract_buffer)
				fatal_error("couldn't allocate space for IPTC extract buffer");
			memcpy(iptc_extract_buffer,
					rewrite_cache+(rewrite_cache_offset-iptc_extract_buffer_size),
					iptc_extract_buffer_size);
			debug("IPTC parser: IPTC chunk extraction prepared from write cache: %lu bytes(s)",
					iptc_extract_buffer_size);
		}
		else
		{
			/* copy ITPC chunk from input flow for later saving */
			iptc_extract_buffer_size=result->parsed_bytes+missing;
			iptc_extract_buffer=(unsigned char*)malloc(iptc_extract_buffer_size*sizeof(unsigned char));
			if (!iptc_extract_buffer)
				fatal_error("couldn't allocate space for IPTC extract buffer");
			memcpy(iptc_extract_buffer, buffer, iptc_extract_buffer_size);
			debug("IPTC parser: IPTC chunk extraction prepared from input flow: %lu bytes(s)",
					iptc_extract_buffer_size);
		}
	}
}

struct iptc_code_descr* iptc_match_code(const enum IPTC_CODE code)
{
	register unsigned int i;

	i=0;
	while (iptc_codes_descr[i].code!=IPTC_EOT)
	{
		if (iptc_codes_descr[i].code==code)
			return(&iptc_codes_descr[i]);
		i++;
	}
	return(NULL);
}

void iptc_dump_list(void)
{
	int i;

	printf("== IPTC datasets ==\n");
	printf("code    description\n");
	for (i=0; iptc_codes_descr[i].code!=IPTC_EOT; i++)
	{
#ifdef IPTC_SHOW_NUMERICAL_ID
		printf("%3d:%03d",	(iptc_codes_descr[i].code&0xff00U)>>8,
				iptc_codes_descr[i].code&0xffU);
#else
		printf("0x%04x ", iptc_codes_descr[i].code);
#endif
		printf(" %s\n", iptc_codes_descr[i].label);
	}
}

void iptc_filter_table_init(void)
{
	unsigned int i;

	iptc_filter_table_size=100;
	iptc_filter_table_filters=0;

	iptc_filter_table=malloc(iptc_filter_table_size*sizeof(struct iptc_filter_match*));
	if (!iptc_filter_table)
		fatal_error("couldn't allocate space for IPTC filter table");

	for (i=0; i<iptc_filter_table_size; i++)
		iptc_filter_table[i]=NULL;
}

void iptc_filter_add_value(unsigned int code)
{
	struct iptc_filter_match* iptc_filter;

	debug("iptc_add_filter_value(%u/%u): 0x%04x", iptc_filter_table_filters,
			iptc_filter_table_size, code);

	/* TODO: dynamically realloc filter table */
	if (iptc_filter_table_filters+1>=iptc_filter_table_size)
		fatal_error("IPTC filter table is full");

	iptc_filter=malloc(sizeof(struct iptc_filter_match));
	if (!iptc_filter)
		fatal_error("couldn't allocate space for IPTC filter value");

	iptc_filter->type=IPTC_FET_VALUE;
	iptc_filter->code=code;
	iptc_filter_table[iptc_filter_table_filters++]=iptc_filter;
}

void iptc_filter_add_range(unsigned int code, unsigned int code2)
{
	struct iptc_filter_match* iptc_filter;

	debug("iptc_add_filter_range(%u/%u): 0x%04x-%04x", iptc_filter_table_filters,
			iptc_filter_table_size, code, code2);

	/* TODO: dynamically realloc filter table */
	if (iptc_filter_table_filters+1>=iptc_filter_table_size)
		fatal_error("IPTC filter table is full");

	iptc_filter=malloc(sizeof(struct iptc_filter_match));
	if (!iptc_filter)
		fatal_error("couldn't allocate space for IPTC filter range");

	iptc_filter->type=IPTC_FET_RANGE;
	iptc_filter->code=code;
	iptc_filter->code2=code2;
	iptc_filter_table[iptc_filter_table_filters++]=iptc_filter;
}

void iptc_filter_table_reset(void)
{
	register unsigned int i;

	for (i=0; i<iptc_filter_table_filters; i++)
	{
		free(iptc_filter_table[i]);
		iptc_filter_table[i]=NULL;
	}
	free(iptc_filter_table);
	iptc_filter_table=NULL;
	debug("iptc_filter_table_reset: %lu IPTC filter(s) unregistered", i);
}

bool iptc_filter_match(unsigned int code)
/* returns true if the IPTC chunk identified by 'code' has to be kept (false if it has
   to be filtered out */
{
	register unsigned int i;
	bool matched;
	bool keep;

	i=0;
	matched=false;
	while (i<iptc_filter_table_filters)
	{
		if (iptc_filter_table[i]->type==IPTC_FET_VALUE)
		{
			if (iptc_filter_table[i]->code==code)
			{
				matched=true;
				break;
			}
		}
		else
			if ((iptc_filter_table[i]->code<=code) && (code<=iptc_filter_table[i]->code2))
			{
				matched=true;
				break;
			}
		i++;
	}
	keep=prefs.filter_exclude?!matched:matched;
	debug("IPTC filter: code 0x%04x to %s (filter mask %smatched)",
			code, keep?"keep":"filter out", matched?"":"not ");
	return(keep);
}

unsigned short int iptc_filters(void)
{
	return(iptc_filter_table_filters);
}
