#ifndef _util_h_
#define _util_h_


#include <stdio.h>
#include <stdarg.h>

#include "config.h"

#define MIN(X,Y) ((X)>(Y)?(Y):(X))
#define MAX(X,Y) ((X)>(Y)?(X):(Y))

#ifndef ssize_t
#define ssize_t long int
#endif

#ifdef OS_DARWIN
#include "gnu-getline.h"
#include "gnu-strndup.h"
#endif

#ifdef OS_QNX
#include "gnu-getline.h"
#include "gnu-strndup.h"
#endif

#ifdef OS_SUNOS
#include "gnu-getline.h"
#include "gnu-strndup.h"
#endif

#ifdef OS_HPUX
#include "gnu-getline.h"
#include "gnu-strndup.h"
#endif

#ifdef OS_AIX
#include "gnu-getline.h"
#include "gnu-strndup.h"
#endif

#ifdef OS_WIN32
#ifndef lstat
#define lstat(x, y) stat(x, y)
#endif
#ifndef S_ISLNK
#define S_ISLNK(m) (0)
#endif
#ifndef S_ISREG
#define S_ISREG(m) ((m&_S_IFREG)==_S_IFREG)
#endif
#ifndef S_ISDIR
#define S_ISDIR(m) (((m)&_S_IFDIR)==_S_IFDIR)
#endif
#ifndef O_CREAT
#define O_CREAT _O_CREAT
#endif
#ifndef O_BINARY
#define O_BINARY _O_BINARY
#endif
#ifndef O_EXCL
#define O_EXCL _O_EXCL
#endif
#include "gnu-getline.h"
#include "gnu-mkstemp.h"
#include "gnu-strcasecmp.h"
#include "gnu-strndup.h"
#endif

#ifdef OS_WIN32
#define DIR_SEPARATOR_STRING "\\"
#define DIR_SEPARATOR_CHAR '\\'
#else
#define DIR_SEPARATOR_STRING "/"
#define DIR_SEPARATOR_CHAR '/'
#endif

#define GIF87_MARKER_STRING "GIF87"
#define GIF89_MARKER_STRING	"GIF89"
#define BMP_MARKER_STRING	"BM"
#define PNG_MARKER_STRING 0x89504e47
#define TIFF_LITTLE_ENDIAN_MARKER_STRING 0x49492a00
#define TIFF_BIG_ENDIAN_MARKER_STRING 0x4d4d002a

typedef short int bool;
#define false 0
#define true 1

enum OP /* reflect changes in */
{
	OP_CHECK,
	OP_DUMP,
	OP_DUMP_FULL,
	OP_FILTER,
	/* reserved */
	OP_EOT
};

#define MIN_LOG_LEVEL 0
#define MAX_LOG_LEVEL 4

#define DEFAULT_WRAP_DUMP 0
#define MIN_WRAP_DUMP 8
#define MAX_WRAP_DUMP 1024

enum LOG_LEVEL
{
	LOG_LEVEL_QUIET,
	LOG_LEVEL_ERRORS,
	LOG_LEVEL_WARNINGS,
	LOG_LEVEL_INFO,
	LOG_LEVEL_DEBUG,
	/* reserved */
	LOG_LEVEL_EOT
};

enum SEVERITY
{
	SEV_FATAL_ERROR,
	SEV_ERROR,
	SEV_WARNING,
	SEV_INFO,
	SEV_DEBUG
};

struct op_label
{
	enum OP	op;
	char*	label;
	char*	description;
};

struct log_level_descr
{
	enum LOG_LEVEL	level;
	char*			description;
};

struct prefs_struct
{
	int log_level;
	bool technical;
	enum OP op;
	char* context_text;
	unsigned long int context_warnings;
	unsigned long int context_errors;
	unsigned long int context_fixes;
	unsigned long int warnings;
	unsigned long int errors;
	unsigned long int fixes;
	unsigned short int wrap_dump;
	bool rewrite;
	bool rewrite_create_backup;
	bool fix;
	bool filter_exclude;
	bool rewrite_cached;
	bool extract_iptc;
	bool extract_extension_append;
	bool test_rewrite;
};

struct parser_result
{
	unsigned long int parsed_bytes;
	unsigned long int written_bytes;
	bool ret;
};

enum EXIT_CODE
{
	EXIT_CODE_NORMAL=0,
	EXIT_CODE_USAGE_ERROR,
	EXIT_CODE_ASYNCHRONOUS_SIGNAL,
	EXIT_CODE_NORMAL_WITH_WARNINGS,
	EXIT_CODE_NORMAL_WITH_ERRORS,
	EXIT_CODE_FATAL_ERROR,
	/* reserved */
	EXIT_EOT
};

struct exit_code_label
{
	enum EXIT_CODE	code;
	char*			label;
};

size_t fsize(FILE*);
bool fexist(const char*);
int fremove(const char*, const char*);
bool fisfile(const char*);
bool fislink(const char*);
bool fisdir(const char*);
char* fextension(const char*);
char* fbasename(const char*);
char* ffilename(const char*);
char* fdirname(const char*);
char* fcleanpath(const char*);

unsigned char getbyte(unsigned char*, unsigned long int);
unsigned short int getword(unsigned char*, unsigned long int);
unsigned long int getlong(unsigned char*, unsigned long int);
unsigned char* getstring(unsigned char*, unsigned long int, unsigned char*, const unsigned long int);
unsigned char* getraw(unsigned char*, unsigned long int, unsigned char*, const unsigned long int);
void setbyte(unsigned char*, unsigned long int, unsigned char);
void setword(unsigned char*, unsigned long int, unsigned short int);
void setlong(unsigned char*, unsigned long int, unsigned long int);

void message_fatal_error(char*, va_list);
void message_error(char*, va_list);
void message_warning(char*, va_list);
void message_info(char*, va_list);
void message_debug(char*, va_list);
void fatal_error(char*, ...);
void error(char*, ...);
void warning(char*, ...);
void info(char*, ...);
void debug(char*, ...);
void message(enum SEVERITY, char*, ...);

void context_set_text(char*);
void context_print_text(void);
void context_print_info(void);
void context_reset(void);

struct op_label* op_match_label(char*);

void init_rewrite();
size_t dump_rewrite(unsigned char*, unsigned long int);
void post_rewrite(unsigned char*, struct parser_result*);

bool get_unsigned_value(const char*, unsigned long int*);
bool get_hexa_value(const char*, unsigned long int*);

size_t strpos(const char*, const char);
size_t strrpos(const char*, const char);

void filter_load(FILE*);

void backup_file(const char*);

void dump_hexa(unsigned char*, unsigned long int);

#endif
